function [final_labels] = CmpDispMulti1(all_images,t_c,s_c,params_PhSR, iteration)
%UNTITLED2 Summary of this function goes here
%   Detailed explanation goes here

% Parameters
UseParallelToolbox = params_PhSR.UseParallelToolbox;
numDisp = params_PhSR.disp_range_small;
r = params_PhSR.filter_kernel_small;
eps = params_PhSR.eps;
thresColor = params_PhSR.thresColor;
thresGrad = params_PhSR.thresGrad;
gamma = params_PhSR.gamma; 
threshBorder = params_PhSR.threshBorder;
gamma_c = params_PhSR.gamma_c; 
gamma_d = params_PhSR.gamma_d;
r_median = params_PhSR.r_median;
mask = fspecial('Gaussian',params_PhSR.maskSize,params_PhSR.maskSigma);

cam_num = 8;
displacement = {[1,-1],[0,-1],[-1,-1],[1,0],[-1,0],[1,1],[0,1],[-1,1]};
st_id = {[-1, 1],[-1, 0],[-1, -1],[0, 1],[0, -1],[1, 1],[1, 0],[1, -1]};
cam_w = ones(cam_num,1)/cam_num;

im_0 = all_images{t_c, s_c};
im_0 = imresize(im_0, params_PhSR.downSampling, 'bilinear');
im_ref = cell(cam_num, 1);

for i = 1:cam_num
    im_ref{i} = all_images{t_c+st_id{i}(1), s_c+st_id{i}(2)};
    im_ref{i} = imresize(im_ref{i}, params_PhSR.downSampling, 'bilinear');
end

% Parameter settings
% r = params.filter_kernel;  % filter kernel in eq. (3) has size r \times r
% r = 201;
% eps = 0.0001;           % \epsilon in eq. (3)
% % thresColor = 2/255;     % \tau_1 in eq. (5)
% thresColor = 2/255;     % \tau_1 in eq. (5)
% thresGrad = 1/255;      % \tau_2 in eq. (5)
% gamma = 0.11;           % (1- \alpha) in eq. (5)
% threshBorder = 3/255;   % some threshold for border pixels
% gamma_c = 0.1;          % \sigma_c in eq. (6)
% gamma_d = 9;            % \sigma_s in eq. (6)
% r_median = 50;          % filter kernel of weighted median in eq. (6) has size r_median \times r_median
% mask = fspecial('Gaussian',[3 3],0.00001);

im1 = im_0;
[m,n,c] = size(im1);

dispVol = zeros(m,n,numDisp);
dispVol1 = zeros(m,n,numDisp);
dispVol_sum = zeros(m,n,numDisp);
consistency_array = cell(cam_num, 1);
time_cam = zeros(cam_num,1);

fprintf('compute multi-disp init: ');

for i = 1:cam_num
    tic;
    fprintf('%d. ',i);

    im2 = im_ref{i};
    Il = double(im1) / 255;
    Ir = double(im2) / 255;

    % Convert to grayscale
    Il_g = rgb2gray(Il);
    Ir_g = rgb2gray(Ir);
    Ir_1 = Il;
    Il_1 = Ir;

    % % Mirror image
    if (displacement{i}(1) ~= 0)
        Il_1 = flipdim(Il_1,2);
        Ir_1 = flipdim(Ir_1,2);
    end
    if (displacement{i}(2) ~= 0)
        Il_1 = flipdim(Il_1,1);
        Ir_1 = flipdim(Ir_1,1);
    end

    % Compute gradient in X-direction from grayscale images
    [fx_l,fy_l] = gradient(Il_g);
    [fx_r,fy_r] = gradient(Ir_g);
    fx_l = fx_l+0.5; % To get a range of values between 0 to 1
    fx_r = fx_r+0.5; % To get a range of values between 0 to 1
    fy_l = fy_l+0.5;
    fy_r = fy_r+0.5;

    fx_l = single(round(fx_l*255))/255;
    fx_r = single(round(fx_r*255))/255;
    fy_l = single(round(fy_l*255))/255;
    fy_r = single(round(fy_r*255))/255;

    fx_l_1 = fx_r;
    fx_r_1 = fx_l;
    fy_l_1 = fy_r;
    fy_r_1 = fy_l;
    if (displacement{i}(1) ~= 0)
        fx_l_1 = flipdim(fx_l_1,2);
        fx_r_1 = flipdim(fx_r_1,2);
        fy_l_1 = flipdim(fy_l_1,2);
        fy_r_1 = flipdim(fy_r_1,2);
    end
    if (displacement{i}(2) ~= 0)
        fx_l_1 = flipdim(fx_l_1,1);
        fx_r_1 = flipdim(fx_r_1,1);
        fy_l_1 = flipdim(fy_l_1,1);
        fy_r_1 = flipdim(fy_r_1,1);
    end

    % Create initial cost volume (eq. (5) in the paper) 

    parfor tmpd=1:numDisp        
        d = tmpd -floor(numDisp/2)-1;     
        
        % Truncated SAD of color images for current displacement
        tmp = image_displacement(Ir, d*displacement{i}, threshBorder);
        p_color = abs(tmp - Il);
        p_color = sum(p_color,3)*0.333333333333;
        
        p_color = filter2(mask, p_color);
        p_color = min(p_color,thresColor);

        % Truncated SAD of gradient images for current displacement
        tmp = image_displacement(fx_r, d*displacement{i}, threshBorder);
        p_grad = abs(tmp - fx_l);
        p_grad = filter2(mask, p_grad);
        p_grad = min(p_grad,thresGrad);
        
        
        tmp = image_displacement(fy_r, d*displacement{i}, threshBorder);        
        p_grad_y = abs(tmp - fy_l);
        p_grad_y = filter2(mask, p_grad_y);
        p_grad_y = min(p_grad_y,thresGrad);

        p = gamma*p_color + (1-gamma)*(p_grad+p_grad_y); % Combined color and gradient
        p = guidedfilter_color(Il, double(p), r, eps);
        dispVol(:,:,tmpd) = guidedfilter_color(Il, double(p), r, eps);
        
    end
    
    parfor tmpd=1:numDisp
        % Same for other view
        d = tmpd -floor(numDisp/2)-1;
        tmp1 = image_displacement(Ir_1, d*displacement{i}, threshBorder);
        p1_color = abs(tmp1 - Il_1);
        p1_color = sum(p1_color,3)*0.333333333333;
        p1_color = filter2(mask, p1_color);
        p1_color = min(p1_color,thresColor);

        tmp1 = image_displacement(fx_r_1, d*displacement{i}, threshBorder);
        p1_grad = abs(tmp1 - fx_l_1);
        p1_grad = filter2(mask, p1_grad);
        p1_grad = min(p1_grad,thresGrad);
        
        tmp1 = image_displacement(fy_r_1, d*displacement{i}, threshBorder);
        p1_grad_y = abs(tmp1 - fy_l_1);
        p1_grad_y = filter2(mask, p1_grad_y);
        p1_grad_y = min(p1_grad_y,thresGrad);

        p1 = gamma*p1_color + (1-gamma)*(p1_grad+p1_grad_y); % Combined color and gradient

        p1 = guidedfilter_color(Il_1, double(p1), r, eps);
        if (displacement{i}(1) ~= 0)
            p1 = flipdim(p1,2);
        end
        if (displacement{i}(2) ~= 0)
            p1 = flipdim(p1,1);
        end
        dispVol1(:,:,tmpd) = p1;

    end
    [labels_left,labels_right] = subpix(dispVol,dispVol1);
    consistency = consistent_check(labels_left, labels_right, displacement{i}, numDisp);
    if ((i==1)||(i==8))
        consistency(1:numDisp,1:numDisp,:) = 1;
        consistency(m-numDisp:m,n-numDisp:n,:) = 1;
    end
    if ((i==3)||(i==6))
        consistency(1:numDisp,n-numDisp:n,:) = 1;
        consistency(m-numDisp:m,1:numDisp,:) = 1;
    end
    dispVol(repmat(consistency,[1 1 numDisp])) = 0;
    consistency_array{i} = consistency;
    dispVol_sum = dispVol_sum + dispVol*cam_w(i);
    
    time_cam(i) = toc;
    
    [disp,disp1] = subpix(dispVol,dispVol1);
    disp = (disp)*255/numDisp;
    disp = uint8(disp);
    imwrite(disp,sprintf('%s/l_disp_ite%d_%d.png',params_PhSR.disp_path,iteration,i));
    
    [labels_left,labels_right] = subpix(9/i*dispVol_sum,dispVol_sum);
    disp = labels_left;
    
    disp = (disp)*255/numDisp;
    disp = uint8(disp);
    imwrite(disp,sprintf('%s/l_disp_ite%d_sum_%d.png',params_PhSR.disp_path,iteration,i));
end

fprintf('\n');


consistency_final = ones(m,n);
for i = 1:cam_num
    consistency_final = consistency_final & consistency_array{i};
end
[labels_left,labels_right] = subpix(dispVol_sum,dispVol_sum);
labels_left(consistency_final) = -100;
inputLabels = labels_left;
[final_labels] = fillPixelsReference(Il, inputLabels, gamma_c, gamma_d, 20, numDisp);

disp = final_labels;

disp = (disp)*255/numDisp;
disp = uint8(disp);
imwrite(disp,sprintf('%s/l_disp_ite%d.png',params_PhSR.disp_path,iteration));

final_labels =  final_labels - floor(numDisp/2)-1;
% final_labels_temp = final_labels;

for i = 1:params_PhSR.iter_refine
    fprintf('%d. ',i);
    % disp(abs(disp)>=5) = 0; % remove wrong part

    % right %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp_rf = FastRefineDisp(double(rgb2gray(im_0)),double(rgb2gray(im_ref{4})),final_labels,1,15,params_PhSR);
    final_labels = guidedfilter_color(double(im_0),disp_rf,10, eps);
    
    disp = final_labels;
    disp = (disp+floor(numDisp/2)+1)*255/numDisp;
    disp = uint8(disp);
    imwrite(disp,sprintf('%s/l_disp_ite%d_refine_%d_00.png',params_PhSR.disp_path,iteration, i));
    
    % left %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    disp_rf = FastRefineDisp(double(rgb2gray(MyFlipIM(im_0, 'lr',0 ))),...
        double(rgb2gray(MyFlipIM(im_ref{5}, 'lr',0 ))),MyFlipIM(final_labels, 'lr',0 ),1,15,params_PhSR);
    final_labels = guidedfilter_color(double(MyFlipIM(im_0, 'lr',0 )),disp_rf,10, eps);
    final_labels = MyFlipIM(final_labels, 'lr',0 );
    
    disp = final_labels;
    disp = (disp+floor(numDisp/2)+1)*255/numDisp;
    disp = uint8(disp);
    imwrite(disp,sprintf('%s/l_disp_ite%d_refine_%d_01.png',params_PhSR.disp_path,iteration, i));
    
    % up %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    disp_rf = FastRefineDisp(double(rgb2gray(MyFlipIM(im_0, 'both',1 ))),...
        double(rgb2gray(MyFlipIM(im_ref{2}, 'both',1 ))),MyFlipIM(final_labels, 'both',1 ),1,15,params_PhSR);
    final_labels = guidedfilter_color(double(MyFlipIM(im_0, 'both',1 )),disp_rf,10, eps);
    final_labels = MyFlipIM(final_labels, 'both',1);
    
    disp = final_labels;
    disp = (disp+floor(numDisp/2)+1)*255/numDisp;
    disp = uint8(disp);
    imwrite(disp,sprintf('%s/l_disp_ite%d_refine_%d_02.png',params_PhSR.disp_path,iteration, i));
    
    % down %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    disp_rf = FastRefineDisp(double(rgb2gray(MyFlipIM(im_0, '',1))),...
        double(rgb2gray(MyFlipIM(im_ref{7}, '',1))),MyFlipIM(final_labels, '',1),1,15,params_PhSR);
    final_labels = guidedfilter_color(double(MyFlipIM(im_0, '',1)),disp_rf,10, eps);
    final_labels = MyFlipIM(final_labels, '',1);
    
    disp = final_labels;
    disp = (disp+floor(numDisp/2)+1)*255/numDisp;
    disp = uint8(disp);
    imwrite(disp,sprintf('%s/l_disp_ite%d_refine_%d_03.png',params_PhSR.disp_path,iteration, i));

end

end

