% function example_referenceForCVPR11(testimage,r,eps,thresColor,thresGrad,gamma,threshBorder,gamma_c,gamma_d,r_median,UseParallelToolbox)

% Load input images and set disparity ranges
if (testimage==1)
    Il = single(imread('.\img_stereo\left.ppm')) / 255;
    Ir = single(imread('.\img_stereo\right.ppm')) / 255;
    numDisp = 15; % Maximum disparity range
elseif (testimage==2)
    Il = double(im1) / 255;
    Ir = double(im2) / 255;
    numDisp = params.disp_range; % Maximum disparity range #################
elseif (testimage==3)
    Il = double(imread('.\img_stereo\cones_left.ppm')) / 255;
    Ir = double(imread('.\img_stereo\cones_right.ppm')) / 255;
    numDisp = 59; % Maximum disparity range
elseif (testimage==4)
    Il = double(imread('.\img_stereo\venus_left.ppm')) / 255;
    Ir = double(imread('.\img_stereo\venus_right.ppm')) / 255;
    numDisp = 32; % Maximum disparity range
end

% Convert to grayscale
Il_g = rgb2gray(Il);
Ir_g = rgb2gray(Ir);

% % Mirror image
Il_1 = flipdim(Ir,2);
Ir_1 = flipdim(Il,2);

% Compute gradient in X-direction from grayscale images
fx_l = gradient(Il_g);
fx_r = gradient(Ir_g);
fx_l = fx_l+0.5; % To get a range of values between 0 to 1
fx_r = fx_r+0.5; % To get a range of values between 0 to 1
% imwrite(fx_l,'gradTmpl.pgm');
% imwrite(fx_r,'gradTmpr.pgm');
% fx_l = single(imread('gradTmpl.pgm'))/255;
% fx_r = single(imread('gradTmpr.pgm'))/255;

fx_l = single(round(fx_l*255))/255;
fx_r = single(round(fx_r*255))/255;

% load test_fl_l;
% sum(sum(abs(fx_l - fx_l_t)))

fx_l_1 = flipdim(fx_r,2);
fx_r_1 = flipdim(fx_l,2);

[m,n,c] = size(Il);

dispVol = ones(m,n,numDisp)*(threshBorder);
dispVol1 = ones(m,n,numDisp)*(threshBorder);

% fprintf('Creating initial cost volume.\n')
time=tic;
% Create initial cost volume (eq. (5) in the paper)

for tmpd=1:numDisp
    d = tmpd -floor(numDisp/2)-1;
%     d = -tmpd;
%     d = tmpd;
    % Truncated SAD of color images for current displacement
    tmp = ones(m,n,c)*threshBorder;
    if(d>0)
        tmp(:,(d+1):n,:) = Ir(:,1:n-d,:);
    else
        tmp(:,1:n+d,:) =  Ir(:,1-d:n,:);
    end
    
    p_color = abs(tmp - Il);
    p_color = sum(p_color,3)*0.333333333333;
    p_color = min(p_color,thresColor);
    
    % Truncated SAD of gradient images for current displacement
    tmp = ones(m,n)*threshBorder;
    if(d>0)
    tmp(:,(d+1):n) = fx_r(:,1:n-d);
    else
         tmp(:,1:n+d) = fx_r(:,(1-d):n);
    end
    
    p_grad = abs(tmp - fx_l);
    p_grad = min(p_grad,thresGrad);
    
    p = gamma*p_color + (1-gamma)*p_grad; % Combined color and gradient
    
    % Same for other view
    tmp1 = ones(m,n,c)*threshBorder;
    if(d>0)
        tmp1(:,(d+1):n,:) = Ir_1(:,1:n-d,:);
    else
        tmp1(:,1:n+d,:) =  Ir_1(:,1-d:n,:);
    end
    p1_color = abs(tmp1 - Il_1);
    p1_color = sum(p1_color,3)*0.333333333333;
    p1_color = min(p1_color,thresColor);
    
    tmp1 = ones(m,n)*threshBorder;
    
    if(d>0)
    tmp1(:,(d+1):n) = fx_r_1(:,1:n-d);
    else
         tmp1(:,1:n+d) = fx_r_1(:,(1-d):n);
    end
    p1_grad = abs(tmp1 - fx_l_1);
    p1_grad = min(p1_grad,thresGrad);
    
    p1 = gamma*p1_color + (1-gamma)*p1_grad; % Combined color and gradient
    
    % Set value in cost volume
    dispVol(:,:,tmpd) = p;
    dispVol1(:,:,tmpd) = flipdim(p1,2);
end

% fprintf('cost volume constructed. using %f sec.\n',toc(time));

% Smooth cost volume with guided filter (using eq. (1) with weights in (4))
if (UseParallelToolbox)
    parfor d=1:numDisp % use parallel for loop when using the parallel computing toolbox
        time = tic;
        p = dispVol(:,:,d);
        p1 = dispVol1(:,:,d);
        
        q = guidedfilter_color(Il, double(p), r, eps);
        p1 =  flipdim(p1,2);
        q1 = guidedfilter_color(Il_1, double(p1), r, eps);
        
        dispVol(:,:,d) = q;
        dispVol1(:,:,d) = flipdim(q1,2);
        fprintf('lable %d filtered. using %f sec.\n',d,toc(time));
    end
else
    for d=1:numDisp % use regular for loop when not using the parallel computing toolbox
        time = tic;
        p = dispVol(:,:,d);
        p1 = dispVol1(:,:,d);
        
        q = guidedfilter_color(Il, double(p), r, eps);
        p1 =  flipdim(p1,2);
        q1 = guidedfilter_color(Il_1, double(p1), r, eps);
        
        dispVol(:,:,d) = q;
        dispVol1(:,:,d) = flipdim(q1,2);
        fprintf('lable %d filtered. using %f sec.\n',d,toc(time));
    end
end

% % Winner take all label selection (eq. (2))
% [unused,labels_left] = min(dispVol,[],3);
% [unused,labels_right] = min(dispVol1,[],3);

[labels_left,labels_right] = subpix(dispVol,dispVol1);
[m,n] = size(labels_left);
%%

% % Left-right consistency check
% Y = repmat((1:m)', [1 n]);

Y = (1:m)'*ones(1,n);

X = repmat(1:n, [m 1]) - round(labels_left-(floor(numDisp/2)+1));
X(X<1) = 1;
X(X>n)=n;
indices = sub2ind([m,n],Y,X);

final_labels = labels_left;
final_labels(abs(labels_left - labels_right(indices))>=1) = -100;

% % Fill and filter (post-process) pixels that fail the consistency check
inputLabels = final_labels;
[final_labels] = fillPixelsReference(Il, inputLabels, gamma_c, gamma_d, 20, numDisp);
final_labels =  final_labels - floor(numDisp/2)-1;
% Show final disparity map
% figure, imagesc(final_labels);
% end
% %%
% fid = fopen('buddha.asc','w');
% for m= 1:768
%     for n=1:768
%       fprintf(fid,'%d,%d,%f\n',m, n,final_labels(m,n)*100);
%     end
% end
        